#!/bin/bash
mode="dev"


if [[ $* =~ ^(prod)$ ]]; then 
	mode="prod"
fi

if [ $mode == "dev" ]; then
    echo "[Installing AURA ENGINE for Development]"
fi
if [ $mode == "prod" ]; then
    echo "[Installing AURA ENGINE for Production]"
fi

# Find the correct Python version (3.7 or 3.8)

if hash python3.9 2>/dev/null; then
	PYTHON_EXEC="python3.9"
	echo "[ Using Python 3.9 ]"
else
	PYTHON_EXEC="python3.8"
	echo "[ Using Python 3.8 ]"
fi

# Development and Production

echo "Installing OPAM Packages ..."
bash scripts/install-opam-packages.sh

echo "Installing Python Requirements ..."
$PYTHON_EXEC $(which pip3) install -r requirements.txt


# Development 

if [ $mode == "dev" ]; then
    echo "Copy configuration to './config/engine.ini'"
    cp -n config/sample-development.engine.ini config/engine.ini
fi


# Production 

if [ $mode == "prod" ]; then
    echo "Copy default Engine configuration to '/etc/aura/engine.ini'"
    cp -n config/sample-production.engine.ini /etc/aura/engine.ini
fi


echo 
echo "+++ Installation of AURA Engine finished! +++"
echo